unit BackupRestore;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, globalroutines, ZConnection, ZSqlProcessor, ZSqlMonitor,
  ZDataset, ZScriptParser, ZCompatibility, ZDbcIntfs, Process, Dialogs, Forms,
  Controls, ComCtrls, LCLIntf, ExtCtrls, StdCtrls;

  (*  ----------------------------------------------------------------------------------------
  gbak:legal switches are:
  -B[ACKUP_DATABASE]    backup database to file
  -BU[FFERS]            override page buffers default
  -C[REATE_DATABASE]    create database from backup file
  -CO[NVERT]            backup external files as tables
  -E[XPAND]             no data compression
  -FA[CTOR]             blocking factor
  -G[ARBAGE_COLLECT)    inhibit garbage collection
  -I[NACTIVE]           deactivate indexes during restore
  -IG[NORE]             ignore bad checksums
  -K[ILL]               restore without creating shadows
  -L[IMBO]              ignore transactions in limbo
  -M[ETA_DATA]          backup metadata only
  -MO[DE] <access>      "read_only" or "read_write" access
  -N[O_VALIDITY]        do not restore database validity conditions
  -NOD[BTRIGGERS]       do not run database triggers
  -NT                   Non-Transportable backup file format
  -O[NE_AT_A_TIME]      restore one table at a time
  -OL[D_DESCRIPTIONS]   save old style metadata descriptions
  -P[AGE_SIZE]          override default page size
  -PAS[SWORD]           Firebird password
  -R[ECREATE_DATABASE]  [O[VERWRITE]] create (replace if O[VERWRITE] used)
                          database from backup file
  -REP[LACE_DATABASE]   replace database from backup file
  -RO[LE]               Firebird SQL role
  -SE[RVICE]            use services manager
  -T[RANSPORTABLE]      transportable backup -- data in XDR format
  -USE_[ALL_SPACE]      do not reserve space for record versions
  -USER                 Firebird user name
  -V[ERIFY]             report each action taken
  -Y  <path>            redirect/suppress status message output
  -Z                    print version number

   The parentheses shown in the above indicates how much of the switch name you need to use
   in order to avoid ambiguity. Once you have specified the absolute minimum - the part before
   the opening '[' - you can use as much of what follows as you wish. For example, to use
   the -b[ackup_database] switch the minimum you must supply on the command line is -b but
   anything between -b and -backup_database will be accepted.

   Using the -help switch like this, or any other invalid switch, will cause gbak to exit
   with an error code of 1 on Linux and Windows.

   https://firebirdsql.org/manual/gbak-cmdline.html
   ------------------------------------------------------------------------------------------- *)



{ **************************************************************************** }
{ * DBBackUpRestore                                                          * }
{ * ------------------------------------------------------------------------ * }
{ *          WhatToDo: 'Backup' oder 'Restore'                               * }
{ *             ZConn: Datenbankverbindung der TZConnection                  * }
{ *        BackupFile: Speicherort des Backup, wenn leer dann DB-Directory   * }
{ * CreateMonitorFile: soll eine Monitoringdatei angelegt werden             * }
{ **************************************************************************** }
procedure DBBackUpRestoreEmb(const WhatToDo: string;
                             var ZConn: TZConnection;
                             var BackupFile: string;
                             var CreateMonitorFile: boolean);


implementation

{ **************************************************************************** }
{ * DBBackUpRestore                                                          * }
{ * ------------------------------------------------------------------------ * }
{ *          WhatToDo: 'Backup' oder 'Restore'                               * }
{ *             ZConn: Datenbankverbindung der TZConnection                  * }
{ *        BackupFile: Speicherort des Backup, wenn leer dann DB-Directory   * }
{ * CreateMonitorFile: soll eine Monitoringdatei angelegt werden             * }
{ **************************************************************************** }
procedure DBBackUpRestoreEmb(const WhatToDo: string;
                             var ZConn: TZConnection;
                             var BackupFile: string;
                             var CreateMonitorFile: boolean);

var                i: integer;
            AProcess: TProcess;
         BatFileText: TStringList;

               s, s1,
        DatabaseName,
             DatFile,
             BakFile,
             OldFile,
        GbackExePath,
             Command,
         GbakCommand: string;
              DBPath: string;

begin

  { Verschiedene Schreibweisen für Backup oder Restore egalisieren }
  s:= LeftStr(WhatToDo,1);
  Command:= UpperCase(s);

  // Während der Datensicherung kann Datenbank bei gbak aktiv bleiben
  // Bei einer Datenrücksicherung Datenbankaktivitäten einstellen
  if (Command='R') then ZConn.Disconnect;

  // Datenbankpfad bzw. Programmpfad: Pfad und Dateinamen bei embedded gleich
  DBPath:= ExtractFilePath(ZConn.Database);
  if (RightStr(DBPath,1)<>PathDelim) then DBPath:= DBPath+ PathDelim;

  { Pfad für den Aufruf der gbak.exe zusammen setzen }
  GbackExePath:= ExtractFilePath(ZConn.LibraryLocation);
  if (RightStr(GbackExePath,1)<>PathDelim) then GbackExePath:= GbackExePath+ PathDelim;
  GbackExePath:= GbackExePath+ 'gbak.exe';

  { Name der Datenbank ohne Extension }
  s:= ExtractFileName(ZConn.Database);
  DatabaseName:=ChangeFileExt(s,'');

  { Der Zugriff auf die Datenbank }
  DatFile:= DBPath+ DatabaseName+ '.FDB';

  { Der Zugriff auf die Backup Datei }
  BakFile:= DBPath+ Databasename+ '.FDK';
  { Wenn BackupFile nicht leer ist, dann diesen Pfad zur Backupdatei verwenden }
  if (BackupFile<>'') then BakFile:= BackupFile;

  { Die zuletzt verfügbare Datenbank mit Datum und Zeitstempel sichern vorbereiten }
  s:= DateTimeToStr(Now);
  { alle Punkte und Doppelpunkte und Leerzeichen mit _ ersetzen }
  For i:=0 to Length(s)-1 do begin
    if (s[i]='.') or (s[i]=':') or (s[i]=' ') then s[i]:= '_';
  end;
  // s:= Copy(s,1, Length(s)-3); // keine Sekunden mitnehmen
  OldFile:= DBPath+ DatabaseName+ '_'+ s+ '.FDB';

  { Wenn s leer bleibt, ist ein Fehler passiert    }
  s:= '';

  { --- Ist ein RESTORE vorgesehen -------------------------------------- }
  if (Command='R') then begin

    { Command store in GbakCommand for BAT and LOG Fiel }
    GBakCommand:= 'RESTORE';

    { alte Datenbank mit Datum und Zeit sicherstellen }
    if (FileExists(BakFile)) then begin
      if  (FileExists(DatFile)) then RenameFile(DatFile, OldFile);
    end else begin
      ShowMessage(' ERROR, there is no existing Backup-File: '+ BakFile);
      Exit;
    end;

    { Log-Datei oder keine Ausgabe während der Datensicherung }
    if (CreateMonitorFile) then s:= ' -y FB'+ GBakCommand+ '.LOG '
                           else s:= ' -y suppress ';

    { Rücksicherungsparameter zusammenstellen }
    s:= ' -R'+  // -replace          replace database from backup file
        ' -z'+  // -z                print version number
        ' -v'+  // -verify           report each action taken
        ' -i'+  // -inactive         deactivate indexes during restore
          s +'-user '+ ZConn.User+ ' -password '+ ZConn.Password+ ' '+
        BakFile+ ' '+ DatFile;
  end;

  { --- Ist ein BACKUP vorgesehen --------------------------------------- }
  if (Command='B') then begin

    { Command store in GbakCommand for BAT and LOG Fiel }
    GBakCommand:= 'BACKUP';

    { Log-Datei oder keine Ausgabe während der Datensicherung Parameter -y }
    if (CreateMonitorFile) then s:= ' -y FB'+ GBakCommand+ '.LOG '
                           else s:= ' -y suppress ';

    { Rücksicherungsparameter zusammenstellen }
    s:= ' -b'+  // -backup_database  backup database to file
        ' -g'+  // -garbage_collect  inhibit garbage collection
        ' -z'+  // -z                print version number
        ' -v'+  // -verify           report each action taken
        ' -ig'+ // -ignore           ignore bad checksums
        s+ '-user '+ ZConn.User+ ' -password '+ ZConn.Password+ ' '+
        DatFile+ ' '+ BakFile;
  end;

  { Wenn der Befehlsstring leer ist, ist etwas schiefgelaufen }
  if (s='') then begin
    ShowMessage(' Backup/Restore is not completed');
    Exit;
  end;

  { --- Der komplette Aufruf für Backup/Restore --------------------------- }
  s:= GbackExePath+ s;

  { BATCH Datei für Sicherung oder Backup kreieren }
  BatFileText:= TStringList.Create;
  BatFileText.Add('ECHO OFF');
  BatFileText.Add('ECHO .');

  { Restore Mitteilungen kreiieren }
  if (Command='R') then begin
    BatFileText.Add('ECHO ***********************************************************');
    BatFileText.Add('ECHO                        R E S T O R E');
    BatFileText.Add('ECHO ***********************************************************');
    BatFileText.Add('ECHO .');
    BatFileText.Add(s);  // Restore Befehl für Ausführen
    BatFileText.Add('ECHO .');
    BatFileText.Add('ECHO -----------------------------------------------------------');
    BatFileText.Add('ECHO '+ DatFile);
    BatFileText.Add('ECHO was renamed to ');
    BatFileText.Add('ECHO '+ OldFile);
    BatFileText.Add('ECHO -----------------------------------------------------------');
    BatFileText.Add('ECHO Restore from:');
    BatFileText.Add('ECHO '+BakFile);
    BatFileText.Add('ECHO Restore to:');
    BatFileText.Add('ECHO '+DatFile);
    BatFileText.Add('ECHO -----------------------------------------------------------');
  end;

  { Backup Mitteilungen kreiieren }
  if (Command='B') then begin
    BatFileText.Add('ECHO ***********************************************************');
    BatFileText.Add('ECHO                        B A C K U P');
    BatFileText.Add('ECHO ***********************************************************');
    BatFileText.Add('ECHO .');
    BatFileText.Add(s);  // Backup Befehl für Ausführen
    BatFileText.Add('ECHO .');
    BatFileText.Add('ECHO -----------------------------------------------------------');
    BatFileText.Add('ECHO '+ BakFile);
    BatFileText.Add('ECHO was created from ');
    BatFileText.Add('ECHO '+ DatFile);
    BatFileText.Add('ECHO -----------------------------------------------------------');
  end;

  BatFileText.Add('ECHO .');
  BatFileText.Add('ECHO -----------------------------------------------------------');

  { Anzeigen, ob Logfile erstellt worden ist }
  if (CreateMonitorFile=True) then begin
    s:= '   A Logfile FB'+ GBakCommand+ '.LOG was created!';
  end else begin
    s:= '   No Logfile was created!';
  end;

  BatFileText.Add('ECHO '+ s);
  BatFileText.Add('ECHO -----------------------------------------------------------');
  BatFileText.Add('ECHO .');
  BatFileText.Add('ECHO .');
  BatFileText.Add('PAUSE');

  { Die BAT Datei speichern }
  try
    s:= DBPath+ GBakCommand+ '.BAT';
    BatFileText.SaveToFile(s);
  finally
    BatFileText.Free;
  end;

  { Den aufrufenden Prozess generieren und starten }
  AProcess := TProcess.Create(nil);
  try
    AProcess.Executable := DBPath+ GBakCommand+ '.BAT';
    AProcess.Options:=AProcess.Options-[poWaitOnExit];
    AProcess.Execute;
    while AProcess.Active do Application.ProcessMessages;
  finally
    AProcess.Free;
  end;

  { Backup BATCH kann belassen und verwendet werden              }
  { Restore Batchdatei löschen, damit kein Fehler passieren kann }
  if (Command='R') then begin
    s:= DBPath+ GBakCommand+ '.BAT';
    if FileExists(s) then DeleteFile(s);
  end;


  { Wenn eine LOG Datei existiert dann aufrufen/anzeigen }
  if (CreateMonitorFile=True) then begin
    s:= DBPath+ 'FB'+ GBakCommand+ '.LOG';
    OpenDocument(s);
  end;

end;



end.

