unit tutbrmaindata;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, db, ZConnection, ZSqlProcessor, ZSqlMonitor, ZDataset,
  globalroutines, StrHolder { Package RX Tools}, ZScriptParser, ZCompatibility,
  ZDbcIntfs, Dialogs;

type

  { TTutBRDataModule }

  TTutBRDataModule = class(TDataModule)
    DataSource1: TDataSource;
    DataSource2: TDataSource;
    StrHolder1: TStrHolder;
    StrHolder2: TStrHolder;
    ZConnectionMain: TZConnection;
    ZQuery1: TZQuery;
    ZQuery1ADRESSID: TLargeintField;
    ZQuery1ADRESSLINE1: TStringField;
    ZQuery1ADRESSLINE2: TStringField;
    ZQuery1CITY: TStringField;
    ZQuery1COMPANY: TStringField;
    ZQuery1COUNTRY: TStringField;
    ZQuery1FORENAME: TStringField;
    ZQuery1LOOKUP: TStringField;
    ZQuery1MIDDLENAME: TStringField;
    ZQuery1POSTCODE: TStringField;
    ZQuery1SALUTATION: TStringField;
    ZQuery1SURNAME: TStringField;
    ZQuery1TITLE: TStringField;
    ZQuery2: TZQuery;
    ZQuery2ADRESSID: TLargeintField;
    ZQuery2CONTACTID: TLargeintField;
    ZQuery2CONTENT1: TStringField;
    ZQuery2CONTENT2: TStringField;
    ZQuery2TITLE: TStringField;
    ZQuery3CONTACTPOSSIBILITYID: TLargeintField;
    ZQuery3CONTACTTITLE: TStringField;
    ZSQLMonitor1: TZSQLMonitor;
    ZSQLProcessor1: TZSQLProcessor;


    procedure DataModuleCreate(Sender: TObject);
    procedure QueryRefresh(qry: TZQuery);
    procedure BeforeDelete(DataSet: TDataSet);
    procedure ZQuery2AfterInsert(DataSet: TDataSet);
    procedure ZQuery2AfterPost(DataSet: TDataSet);
    procedure ZQuery1AfterPost(DataSet: TDataSet);
    procedure ZQuery1BeforeDelete(DataSet: TDataSet);

    function  ZConnectionInit(var ZConn: TZConnection): boolean;
    function  CreateDatabase(var ZConn: TZConnection): boolean;
  private

  public

  end;

// const FirebirdDatabase: string = 'FBDATA.FDB';


var   TutBRDataModule: TTutBRDataModule;


{ Für dieses Tutorial sind die Ausgangsdaten als Konstante definiert }
CONST FirebirdDir: string = 'fb464';
      FirebirdClient: string = 'fbclient.dll';
      FirebirdProtocol: string = 'firebird';
      FirebirdClientCodePage: string = 'UTF8';
      FirebirdPassword: string = 'masterkey';
      FirebirdUser: string = 'SYSDBA';
      FirebirdDatabase: string = 'TUTBACKTEST.FDB';


implementation

{$R *.lfm}


{ **************************************************************************** }
{ * Alles, was möglich ist beim Start von DataModule initialsieren           * }
{ **************************************************************************** }
procedure TTutBRDataModule.DataModuleCreate(Sender: TObject);

var s: string;

begin
  { Die Properties von ZConnection initialsieren }
  TutBRDataModule.ZConnectionMain.Disconnect;
  TutBRDataModule.ZConnectionInit(ZConnectionMain);

  { Wenn keine Datenbank vorhanden ist, dann anlegen }
  s:= ZConnectionMain.Database;
  if not fileexists(s) then begin
    CreateDatabase(ZConnectionMain);
  end;

end;

{ **************************************************************************** }
{ * GZConnectionMainInit                                                     * }
{ **************************************************************************** }
function TTutBRDataModule.CreateDatabase(var ZConn: TZConnection): boolean;

var s: string;
   ok: boolean;

begin

  { Kontrolliert, ob die Funktion vor Erreichen des Endes verlassen wird }
  ok:= false;

  with ZConn do begin

    { Leere Datenbank anlegen }
    Properties.Clear;
    { Daten werden von der initialsierten ZConnection übernommen }
    Properties.Add ('CreateNewDatabase=CREATE DATABASE ' +
                    QuotedStr (Database) +
                    ' USER ' + QuotedStr(User) +
                    ' PASSWORD ' + QuotedStr(Password) +
                    ' PAGE_SIZE 8192 DEFAULT CHARACTER SET '+ ClientCodePage);
    { Verbinden mit der Datenbank erzeugt diese, dann Datenbank schließen }
    Connect;
    { Datenbank schließen und Properties leeren }
    Disconnect;
    Properties.Clear;

    { ZSQLProcessor mit ZConnection verbinden }
    ZSQLProcessor1.Connection:= ZConn;

    { === Den ersten SQL Script starten ===}
    ZSQLProcessor1.Script.Clear;
    ZSQLProcessor1.Script:= StrHolder1.Strings;
    ZSQLProcessor1.DelimiterType:= TZDelimiterType(0);
    Connect;
    if ZSQLProcessor1.Script.Count>0 then ZSQLProcessor1.Execute;

    // Verbindung zur Datenbank trennen
    Disconnect;

    { === Den zweiten SQL Script starten ===}
    ZSQLProcessor1.Script.Clear;
    ZSQLProcessor1.Script:= StrHolder2.Strings;
    ZSQLProcessor1.DelimiterType:= TZDelimiterType(4);
    Connect;
    if ZSQLProcessor1.Script.Count>0 then ZSQLProcessor1.execute;

    // Verbindung zur Datenbank trennen
    Disconnect;

    // Delimiter auf Standardwert zurücksezten
    ZSQLProcessor1.Script.Clear;
    ZSQLProcessor1.DelimiterType:= TZDelimiterType(0);

    { Datenbank wurde angelegt                  }
    { Database is created                       }
    if fileexists(Database) = true then begin
      s:= 'Database: '+ LineEnding+ Database+ ' is created!';
      ShowMessage(s);
    end;

  end;

  ok:= True;
  CreateDatabase:= ok;

end;

{ **************************************************************************** }
{ * GZConnectionMainInit                                                     * }
{ **************************************************************************** }
function TTutBRDataModule.ZConnectionInit(var ZConn: TZConnection): boolean;

var s, AppPath: string;

begin

  { Der Pfad der Applikation }
  GetApplicationPath(AppPath);

  with ZConn do begin

    { Die Properties der #Reihe nach }
    AutoCommit:= True;
    // Catalog:=
    ClientCodePage:= FirebirdClientCodePage;
    // Connected
    // ControlsCodePage cCP_UTF8 eingestellt

    { Database Location }
    s:= AppPath+ FirebirdDatabase;
    DataBase:= s;

    // DesignConnection
    // Hostname:= 'localhost';

    { Library Location - fbclient.dll }
    s:= AppPath+ FirebirdDir+ PathDelim+ FirebirdClient;
    LibraryLocation:= s;

    // LoginPrompt
    // Name
    Password:= FirebirdPassword;
    // Port
    // Properties
    Protocol:= FirebirdProtocol;
    // ReadOnly
    // SQLHourGlass
    // Tag
    // TransactionIsolationLevel  tRedCommitted
    // UseMetaDate True
    User:= FirebirdUser;
    // Version
    ZConnectionInit:= True;

  end;

end;


{ **************************************************************************** }
{ * QueryRefresh                                                             * }
{ **************************************************************************** }
procedure TTutBRDataModule.QueryRefresh(qry: TZQuery);

Var  bm: TBookmark;

begin
  { übergebene ZQuery mit with generell zugängig machen }
  { make passed ZQuery generally accessible with        }
  with qry do begin
    // Die Position des neu angelegten Datensatze im Bookmark bm merken
    // memorize the position of the newly created record in Bookmark bm
    bm:= Bookmark;
    { in einen try/except Block packen                         }
    { damit die Verbindung zur Anzeige wieder hergestellt wird }
    { pack in a try / except block to reconnect to the ad      }
    try
      { keine Verbindung zu den Anzeigeelementen, spart Zeit und ruckelt nicht }
      { no connection to the display elements, saves time and does not jerk    }
      DisableControls;
      { Die Daten werden neu eingelesen, damit kann der neu erstellte Datensatz }
      { auch für Verknüpfungen verwendet werden                                 }
      { Es würde jetzt der erste Datensatz der Datenmenge angezeigt             }
      { The data will be read in again so that the newly created data set, also }
      { used for shortcuts, the first record of the dataset would now be        }
      { displayed                                                               }
      Refresh;
      { und den Datensatzzeiger auf den gerade angelegten Datensatz legen }
      { and place the record pointer on the record you just created       }
      Bookmark:= bm;
      { Sollte bis hier etwas schief gehen wird auf jeden Fall die Daten wieder }
      { mit der Anzeige verbunden                                               }
      { Should something go wrong here, the data will definitely be back        }
      { connected to the ad                                                     }

    finally
      { Verbindung herstellen }
      {Connect }
      EnableControls;
    end;
  end;
end;


{ **************************************************************************** }
{ * TDataDMTutor2.ZQuery2AfterInsert zum schnellen Datenerfassen             * }
{ **************************************************************************** }
procedure TTutBRDataModule.ZQuery2AfterInsert(DataSet: TDataSet);

begin
     // Bei jedem neu angelegten Datensatz wird der Text 'Email' in Title erfasst
     // if ZQuery2.State in [dsEdit, dsInsert] then
     //     ZQuery2.FieldByName('TITLE').AsString:= 'Handy'; // Email ...
end;


{ **************************************************************************** }
{ * TDataDMTutor2.ZQuery2AfterPost(DataSet: TDataSet)                        * }
{ **************************************************************************** }
procedure TTutBRDataModule.ZQuery2AfterPost(DataSet: TDataSet);

begin
  // Aufruf der Procedure QueryRefresh
  // Call the Procedure QueryRefresh
  QueryRefresh(DataSet as TZQuery);
end;

{ **************************************************************************** }
{ * ZQuery1AfterPost                                                         * }
{ **************************************************************************** }
procedure TTutBRDataModule.ZQuery1AfterPost(DataSet: TDataSet);

//var bi: string;

begin

  // Aufruf der Procedure QueryRefresh
  // Call the Procedure QueryRefresh
  QueryRefresh(DataSet as TZQuery);

  // Abfrage der Nummer des zuletzt eingefügten/geänderten Datensatzes
  // Query the number of the last inserted / changed record

  // bi := Dataset.FieldByName('ADRESSID').AsString;
  // FormMainTutor2.LabelTest.Caption:= bi;

end;

{ **************************************************************************** }
{ * ZQuery1BeforeDelete                                                      * }
{ **************************************************************************** }
procedure TTutBRDataModule.ZQuery1BeforeDelete(DataSet: TDataSet);
begin
  BeforeDelete(ZQuery2);
end;



{ **************************************************************************** }
{ * BeforeDelete                                                             * }
{ **************************************************************************** }
procedure TTutBRDataModule.BeforeDelete(DataSet: TDataSet);

var i: integer;

begin

  { Bevor die Daten von ADRESS gelöscht werden,}
  { müssen die von CONTACT gelöscht werden     }
  with (DataSet as TZQuery) do begin
    { Anzeige vorübergehend blockieren  }
    {Temporarily block ads              }
    DisableControls;
    { Die Anzahl der Datensätze übernehmen }
    { Apply the number of records          }
    i:= RecordCount;
    { Und so oft den Löschvorgang aufrufen, bis alle Datensätze gelöscht sind }
    { And call the deletion until all records are deleted                     }
    First;
    for i:=0 to i-1 do begin
      Delete;
    end;
    { Die Datenanzeige wieder verwenden }
    { Reuse the data display            }
    EnableControls;
  end;
end;

end.

